<p align="center">
    <img src="./examples/assert/logo/VFluent.png"/>
</div>

<p align="center">
    <a href="LICENSE">
      <img src="https://img.shields.io/badge/License-MIT-yellow.svg">
    </a>
    <a href="BUILD">
      <img src="https://travis-ci.com/aleversn/VFluent.svg?branch=master">
    </a>
    <a href="">
      <img src="https://img.shields.io/npm/dw/vfluentdesign">
    </a>
</p>

## Fluent UI Components based on Vue Client
This repository provides a library of components based on [Microsoft's Fluent Design System](https://developer.microsoft.com/en-us/fluentui#/). We did our best to implement styling (such as the `Acrylic` and `Reveal` effects) of native system's (Windows 11/Windows 10) compoents on the Web.

### Project Structure

```bash
- build # 脚手架(The scaffold)
  config.js # 组件库配置文件(Component library configuration file)
- examples
  - docs # 在线演示文档(VuePress)
  index.js # Vue Cli Lib 打包入口(Packaging entrance)
- lib # 打包ump文件(Package the UMP file)
- packages
  - common-theme # 通用主题(Common theme)
  - {themeName}-theme
    index.scss # 全局scss 由脚本生成(Global SCSS generated by the script)
    {ComponentScss}.scss
  - {ComponentName}
    - src # 组件Source(Component source)
      index.vue # 组件(Component)
    index.js # 组件入口(Component entry)
  index.js # 所有组件入口 由脚本生成(All component entries, generated by scripts)
components.json # 组件目录(Component directory)
vue.config.js # Vue cli 配置(Vue cli config)
```

### NPM Script CMDs

```bash
pnpm run pub # 打包推送到npm(Pack and push to NPM)
pnpm run docs:dev # 开发文档模式(Run as development document mode)
pnpm run bin:new componentName chineseName #新建组件(Create new component)
pnpm run bin:rm componentName #移除组件并重新定制入口(Remove the component and re-customize the entry)
pnpm run bin:entry # 定制入口(Custom entry)
```

### Fluent Design UI (Vue 2.7+)

#### DOCS

[Documention](https://aleversn.github.io/VFluent)

#### How to use

1. install via pnpm

```bash
  pnpm i vfluentdesign # recommend
```

2. import (vue entry main.js)

```js
import Vue from "vue";
import Vuex from "vuex"
import VueFluent from "vfluentdesign";
import "vfluentdesign/lib/index.css";

Vue.use(VueFluent, Vuex);
```

3. sample example

```html
<fv-button>Hello Vue Fluent!</fv-button>
```

4. about Fabric UI

Our project have many using cases contains Microsoft's Fabric UI, such as icons and shadows, for more details you can click <a href="https://developer.microsoft.com/en-us/fabric#/styles">here</a> to get more information.

Here is the icon using sample.

```vue
<i class="ms-Icon ms-Icon--AADLogo"></i>
```

In particular, if you're using a component of Fluent Vue Design and it contains a prop that support icon, you only need to type the icon's name in prop.

**We have updated the new Windows 11 Fluent Icons, Check the Icon Dictionary on <a href="https://docs.microsoft.com/en-us/windows/apps/design/style/segoe-fluent-icons-font">here</a>.**

### ✨ We Need You (Become a contributor) ✨
VFluent still needs to improve. The documentation is not yet complete, there are still some details to work out.

We want to hear from your issues and suggestions. It's welcome to have you become a contributor in this project. You could add some new components to VFluent, or update the existing components to better support the mobile end.

If you have some other creative ideas, we're happy to hear them from you.

### License

MIT License

Copyright (c) 2023 Creator SN®

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

### Status

---

![Alt](https://repobeats.axiom.co/api/embed/c3151fa6bc7f4329d5d136aff5300b5a858f8b67.svg "Repobeats analytics image")
